import base64
import subprocess
import uuid
import winreg

from cryptography.fernet import Fernet

__all__ = ['get_machine_user_fernet']

__key = uuid.UUID(winreg.QueryValueEx(winreg.OpenKey(key=winreg.HKEY_LOCAL_MACHINE,
                                                     sub_key=r"SOFTWARE\\Microsoft\\Cryptography\\"),
                                      'MachineGuid')[0]).bytes
__salt = uuid.UUID(int=int(subprocess.check_output('whoami /User').decode().split('\n')[6].split()[1][1:].replace('-',''))).bytes


def get_machine_user_fernet() -> Fernet:
    """Return Fernet object based on machine and user"""
    return Fernet(base64.urlsafe_b64encode(__key + __salt))



import pathlib
import sys
if sys.platform == 'win32':
    from .win32 import get_machine_user_fernet
else:
    from .linux import get_machine_user_fernet

from hakisto import Logger
logger = Logger('imuthes.crypt')

__all__ = ['enigma', 'denigma', 'read']



def enigma(value: str) -> bytes:
    """Return ``value`` encrypted for machine and user.

    :param value: String to be encrypted.
    :type value: ``str``
    :returns: Encrypted string.
    :rtype: ``bytes``
    """
    logger.verbose("Encrypting string")
    return get_machine_user_fernet().encrypt(value.encode())


def denigma(value: bytes) -> str:
    """Return ``value`` decrypted for machine and user.

    :param value: Encrypted string.
    :type value: ``bytes``
    :returns: Decrypted string.
    :rtype: ``str``
    """
    logger.verbose("Decrypting string")
    return get_machine_user_fernet().decrypt(value).decode()


def read(path: pathlib.Path) -> str:
    with path.open('rb') as f:
        return denigma(f.read())



import pathlib
import click
from click_default_group import DefaultGroup
from .crypt import enigma
from hakisto.click import hakisto_inline_location, hakisto_severity, hakisto_short_trace, hakisto_process_all


@click.group(context_settings=dict(help_option_names=["-h", "--help"]))
@hakisto_severity()
@hakisto_inline_location()
@hakisto_short_trace()
@click.pass_context
def main(ctx, **kwargs):
    """Crypt utilities"""
    hakisto_process_all(**kwargs)
    ctx.ensure_object(dict)


@main.group(cls=DefaultGroup,
        default="store",
        default_if_no_args=True,)
@click.option("-f", "--file", type=click.Path(dir_okay=False, path_type=pathlib.Path), prompt=True)
@click.pass_context
def secret(ctx, file):
    """Locally stored secret"""
    ctx.obj["file"] = file


@secret.command()
@click.argument("phrase")
@click.pass_context
def store(ctx, phrase):
    """Store secret PHRASE"""
    with ctx.obj["file"].open('wb') as f:
        f.write(enigma(phrase))




import base64
import hashlib
import os
import pathlib
import uuid

from cryptography.fernet import Fernet

from hakisto import Logger
logger = Logger('kion25.crypt.linux')

__all__ = ['get_machine_user_fernet']


def ensure_path(file_name: str) -> pathlib.Path:
    p = pathlib.Path.home() / ".local"
    p.mkdir(exist_ok=True, parents=True)
    p = p / file_name
    if not p.exists():
        logger.info("Creating random id")
        with p.open("w") as f:
            f.write(os.urandom(16).hex())
    return p


path = pathlib.Path("/etc/machine-id")
if not path.exists():
    logger.warning("Could not find /etc/machine-id")
    path = ensure_path("machine-id")
with path.open() as f:
    __key = uuid.UUID(f.read().strip()).bytes

try:
    uid = os.getuid()
except OSError:
    logger.warning("Issue getting User ID")
    path = ensure_path("user-id")
    with path.open() as f:
        __salt = uuid.UUID(f.read().strip()).bytes
else:
    md5 = hashlib.md5()
    md5.update((os.getlogin() * (uid % (2 ** 16) or 1)).encode())
    __salt = md5.digest()

def get_machine_user_fernet() -> Fernet:
    """Return Fernet object based on machine and user"""
    return Fernet(base64.urlsafe_b64encode(__key + __salt))
